@@ -1,6 +1,6 @@
 /* ACVP proxy protocol handler for managing the operational env information
  *
- * Copyright (C) 2018 - 2019, Stephan Mueller <smueller@chronox.de>
+ * Copyright (C) 2018 - 2020, Stephan Mueller <smueller@chronox.de>
  *
  * License: see LICENSE file in root directory
  *
@@ -107,6 +107,12 @@ static int acvp_oe_build_dep_sw(const struct def_oe *def_oe,
 	struct json_object *dep = NULL;
 	int ret = -EINVAL;
 
+	/* We are required to have an entry at this point. */
+	if (!def_oe->oe_env_name) {
+		*json_oe = NULL;
+		return 0;
+	}
+
 	/*
 	 * {
 	 *     "type" : "software",
@@ -135,9 +141,7 @@ static int acvp_oe_build_dep_sw(const struct def_oe *def_oe,
 		CKINT(json_object_object_add(dep, "swid",
 				json_object_new_string(def_oe->swid)));
 	} else {
-		logger(LOGGER_ERR, LOGGER_C_ANY, "CPE or SWID missing\n");
-		ret = -EINVAL;
-		goto out;
+		logger(LOGGER_VERBOSE, LOGGER_C_ANY, "No CPE or SWID found\n");
 	}
 
 	if (def_oe->oe_description) {
@@ -145,7 +149,7 @@ static int acvp_oe_build_dep_sw(const struct def_oe *def_oe,
 			json_object_new_string(def_oe->oe_description)));
 	} else {
 		CKINT(json_object_object_add(dep, "description",
-				json_object_new_string("TOBEDEFINED")));
+				json_object_new_string(def_oe->oe_env_name)));
 	}
 
 	json_logger(LOGGER_DEBUG2, LOGGER_C_ANY, dep, "Vendor JSON object");
@@ -164,8 +168,7 @@ static int acvp_oe_add_dep_url(uint32_t id, struct json_object *dep)
 	int ret = -EINVAL;
 	char url[ACVP_NET_URL_MAXLEN];
 
-	CKINT(acvp_create_urlpath(NIST_VAL_OP_DEPENDENCY, url,
-				  sizeof(url)));
+	CKINT(acvp_create_urlpath(NIST_VAL_OP_DEPENDENCY, url, sizeof(url)));
 	CKINT(acvp_extend_string(url, sizeof(url), "/%u", id));
 	CKINT(json_object_array_add(dep, json_object_new_string(url)));
 
@@ -198,6 +201,28 @@ static int acvp_oe_match_dep_sw(struct def_oe *def_oe,
 				     def_oe->acvp_oe_dep_sw_id));
 	}
 
+	/*
+	 * Check for the presence of a SWID/CPE on the server where locally
+	 * there is none defined.
+	 */
+	if (!def_oe->swid && !def_oe->cpe) {
+		ret = json_get_string(json_oe, "swid", &str);
+
+		/* Found one, we have a mismatch */
+		if (!ret) {
+			ret = -ENOENT;
+			goto out;
+		}
+
+		ret = json_get_string(json_oe, "cpe", &str);
+
+		/* Found one, we have a mismatch */
+		if (!ret) {
+			ret = -ENOENT;
+			goto out;
+		}
+	}
+
 	CKINT(json_get_string(json_oe, "description", &str));
 	CKINT(acvp_str_match(def_oe->oe_description, str,
 			     def_oe->acvp_oe_dep_sw_id));
@@ -326,6 +351,9 @@ static int acvp_oe_register_dep(const struct acvp_testid_ctx *testid_ctx,
 		break;
 	}
 
+	if (!json_oe)
+		goto out;
+
 	CKINT(acvp_create_url(NIST_VAL_OP_DEPENDENCY, url, sizeof(url)));
 	CKINT(acvp_meta_register(testid_ctx, json_oe, url, sizeof(url), id,
 				 submit_type));
@@ -368,39 +396,20 @@ static int acvp_oe_validate_one_dep(const struct acvp_testid_ctx *testid_ctx,
 	unsigned int http_type = 0;
 
 	ret = _acvp_oe_validate_one_dep(testid_ctx, def_oe, depid);
-	if (ret && ret != -ENOENT)
+
+	ret = acvp_search_to_http_type(ret, ACVP_OPTS_DELUP_OE, ctx_opts, 0,
+				       &http_type);
+	if (ret < 0) {
+		logger(LOGGER_ERR, LOGGER_C_ANY,
+		       "Conversion from search type to HTTP request type failed for OE dependencies %u/%u\n",
+		       def_oe->acvp_oe_dep_proc_id, def_oe->acvp_oe_dep_sw_id);
+		goto out;
+	}
+
+	if (http_type == acvp_http_none)
 		goto out;
 
-	/* If we did not find a match, update the module definition */
-	if (ret == -ENOENT) {
-		if (ctx_opts->update_db_entry & ACVP_OPTS_DELUP_OE) {
-			http_type = acvp_http_put;
-		} else if (ctx_opts->delete_db_entry &
-			   (ACVP_OPTS_DELUP_OE | ACVP_OPTS_DELUP_FORCE)) {
-			   http_type = acvp_http_delete;
-		} else if (ctx_opts->register_new_oe) {
-			http_type = acvp_http_post;
-		} else {
-			logger(LOGGER_ERR, LOGGER_C_ANY,
-			       "Definition for OE dependency IDs %u/%u different than found on ACVP server - you need to perform a (re)register operation\n",
-			       def_oe->acvp_oe_dep_proc_id,
-			       def_oe->acvp_oe_dep_sw_id);
-			goto out;
-		}
-	/*
-	 * We only attempt a delete if we have a match between the ACVP server
-	 * DB and our configurations. We do not want to delete unknown
-	 * definitions. Yet, if we are forced to perform the delete, we will
-	 * do that.
-	 */
-	} else if (ctx_opts->delete_db_entry & ACVP_OPTS_DELUP_OE) {
-		http_type = acvp_http_delete;
-	}
-
-	if (http_type) {
-		CKINT(acvp_oe_register_dep(testid_ctx, def_oe, type,
-					   http_type));
-	}
+	CKINT(acvp_oe_register_dep(testid_ctx, def_oe, type, http_type));
 
 out:
 	return ret;
@@ -413,18 +422,66 @@ struct acvp_oe_match_struct {
 		      struct def_oe *def_oe, struct json_object *json_oe);
 };
 
+/**
+ * NIST requests the "name" keyword in the OE JSON definition to be unique with
+ * a human readable information about the OS and the processor information.
+ *
+ * We are assembling such a string here but making sure our JSON information
+ * does not keep duplicate information.
+ */
+static int acvp_oe_generate_oe_string(struct def_oe *def_oe, char *str,
+				      size_t stringlen)
+{
+	int ret = 0;
+
+	str[0] = '\0';
+
+	if (def_oe->oe_env_name) {
+		CKINT(acvp_extend_string(str, stringlen, "%s",
+					 def_oe->oe_env_name));
+		if (def_oe->manufacturer || def_oe->proc_series ||
+		    def_oe->proc_name)
+			CKINT(acvp_extend_string(str, stringlen, " on"));
+	}
+
+	if (def_oe->manufacturer)
+		CKINT(acvp_extend_string(str, stringlen, " %s",
+					 def_oe->manufacturer));
+
+	if (def_oe->proc_series) {
+		CKINT(acvp_extend_string(str, stringlen, " %s",
+					 def_oe->proc_series));
+
+		if (def_oe->proc_name &&
+		    strncmp(def_oe->proc_series, def_oe->proc_name,
+			    strlen(def_oe->proc_name))) {
+			CKINT(acvp_extend_string(str, stringlen, " %s",
+						 def_oe->proc_name));
+		}
+	} else {
+		if (def_oe->proc_name)
+			CKINT(acvp_extend_string(str, stringlen, " %s",
+						 def_oe->proc_name));
+	}
+
+out:
+	return ret;
+}
+
 static int acvp_oe_match_oe(const struct acvp_testid_ctx *testid_ctx,
 			    struct def_oe *def_oe, struct json_object *json_oe)
 {
 	struct json_object *tmp;
 	unsigned int i;
 	int ret;
+	char oe_name[FILENAME_MAX];
 	const char *str;
 
 	(void)testid_ctx;
 
+	CKINT(acvp_oe_generate_oe_string(def_oe, oe_name, sizeof(oe_name)));
 	CKINT(json_get_string(json_oe, "name", &str));
-	CKINT(acvp_str_match(def_oe->oe_env_name, str, def_oe->acvp_oe_id));
+	CKINT(acvp_str_match(oe_name, str, def_oe->acvp_oe_id));
 
 	CKINT(json_get_string(json_oe, "url", &str));
 	CKINT(acvp_get_trailing_number(str, &def_oe->acvp_oe_id));
@@ -490,8 +547,7 @@ static int _acvp_oe_validate_all(const struct acvp_testid_ctx *testid_ctx,
 	match_def.def_oe = def_oe;
 	match_def.matcher = matcher;
 
-	CKINT(acvp_paging_get(testid_ctx, url, &match_def,
-			      &acvp_oe_match_cb));
+	CKINT(acvp_paging_get(testid_ctx, url, &match_def, &acvp_oe_match_cb));
 
 out:
 	return ret;
@@ -518,9 +574,8 @@ out:
 	return ret;
 }
 
-static int
-acvp_oe_validate_add_searchopts(const char *searchstr,
-				char *url, unsigned int urllen)
+static int acvp_oe_validate_add_searchopts(const char *searchstr,
+					   char *url, unsigned int urllen)
 {
 	int ret;
 	char queryoptions[384], str[128];
@@ -529,7 +584,8 @@ acvp_oe_validate_add_searchopts(const char *searchstr,
 	 * Set a query option consisting of the dependency name - we OR all
 	 * of them.
 	 */
-	CKINT(bin2hex_html(searchstr, strlen(searchstr), str, sizeof(str)));
+	CKINT(bin2hex_html(searchstr, (uint32_t)strlen(searchstr), str,
+			   sizeof(str)));
 	snprintf(queryoptions, sizeof(queryoptions), "name[0]=contains:%s",
 		 str);
 	CKINT(acvp_append_urloptions(queryoptions, url, urllen));
@@ -561,7 +617,7 @@ static int acvp_oe_validate_all_dep(const struct acvp_testid_ctx *testid_ctx,
 	}
 
 	/* Search for software */
-	if (!def_oe->acvp_oe_dep_sw_id) {
+	if (def_oe->oe_env_name && !def_oe->acvp_oe_dep_sw_id) {
 		CKINT(acvp_create_url(NIST_VAL_OP_DEPENDENCY, url,
 				      sizeof(url)));
 		CKINT(acvp_oe_validate_add_searchopts(def_oe->oe_env_name, url,
@@ -582,13 +638,15 @@ static int acvp_oe_validate_all_dep(const struct acvp_testid_ctx *testid_ctx,
 			goto out;
 	}
 
-	if (!def_oe->acvp_oe_dep_sw_id) {
-		ret |= acvp_oe_register_dep_type(testid_ctx, def_oe,
-						 ACVP_OE_DEP_TYPE_SW);
-	} else if (ctx_opts->delete_db_entry & ACVP_OPTS_DELUP_OE) {
-		ret |= acvp_oe_register_dep(testid_ctx, def_oe,
-					    ACVP_OE_DEP_TYPE_SW,
-					    acvp_http_delete);
+	if (def_oe->oe_env_name) {
+		if (!def_oe->acvp_oe_dep_sw_id) {
+			ret |= acvp_oe_register_dep_type(testid_ctx, def_oe,
+							ACVP_OE_DEP_TYPE_SW);
+		} else if (ctx_opts->delete_db_entry & ACVP_OPTS_DELUP_OE) {
+			ret |= acvp_oe_register_dep(testid_ctx, def_oe,
+						ACVP_OE_DEP_TYPE_SW,
+						acvp_http_delete);
+		}
 	}
 
 out:
@@ -609,6 +667,7 @@ static int acvp_oe_build_oe(const struct acvp_testid_ctx *testid_ctx,
 	struct json_object *oe = NULL, *depurl = NULL, *deparray = NULL,
 			   *dep = NULL;
 	int ret = 0;
+	char oe_name[FILENAME_MAX];
 	bool depadded = false;
 
 	/* Validate dependencies and create JSON request. */
@@ -629,12 +688,16 @@ static int acvp_oe_build_oe(const struct acvp_testid_ctx *testid_ctx,
 				CKINT(json_object_array_add(deparray, dep));
 			}
 			if (!def_oe->acvp_oe_dep_sw_id) {
-				if (!deparray) {
-					deparray = json_object_new_array();
-					CKNULL(deparray, -ENOMEM);
-				}
 				CKINT(acvp_oe_build_dep_sw(def_oe, &dep));
-				CKINT(json_object_array_add(deparray, dep));
+				if (dep) {
+					if (!deparray) {
+						deparray =
+							json_object_new_array();
+						CKNULL(deparray, -ENOMEM);
+					}
+					CKINT(json_object_array_add(deparray,
+								    dep));
+				}
 			}
 		}
 	}
@@ -657,8 +720,9 @@ static int acvp_oe_build_oe(const struct acvp_testid_ctx *testid_ctx,
 
 	oe = json_object_new_object();
 	CKNULL(oe, -ENOMEM);
+	CKINT(acvp_oe_generate_oe_string(def_oe, oe_name, sizeof(oe_name)));
 	CKINT(json_object_object_add(oe, "name",
-			json_object_new_string(def_oe->oe_env_name)));
+				     json_object_new_string(oe_name)));
 	if (depurl) {
 		CKINT(json_object_object_add(oe, "dependencyUrls", depurl));
 		depurl = NULL;
@@ -672,8 +736,7 @@ static int acvp_oe_build_oe(const struct acvp_testid_ctx *testid_ctx,
 
 	if (!depadded) {
 		logger(LOGGER_WARN, LOGGER_C_ANY,
-		       "No dependencies found for OE %s\n",
-		       def_oe->oe_env_name);
+		       "No dependencies found for OE %s\n", oe_name);
 	}
 
 	json_logger(LOGGER_DEBUG2, LOGGER_C_ANY, oe, "Vendor JSON object");
@@ -719,7 +782,7 @@ static int acvp_oe_validate_one_oe(const struct acvp_testid_ctx *testid_ctx,
 	const struct acvp_opts_ctx *ctx_opts = &ctx->options;
 	int ret;
 	char url[ACVP_NET_URL_MAXLEN];
-	unsigned int http_type = 0;
+	enum acvp_http_type http_type;
 
 	CKNULL_LOG(ctx, -EINVAL,
 		   "Vendor validation: authentication context missing\n");
@@ -734,41 +797,17 @@ static int acvp_oe_validate_one_oe(const struct acvp_testid_ctx *testid_ctx,
 
 	ret = _acvp_oe_validate_one(testid_ctx, def_oe, url,
 				    acvp_oe_match_oe);
-	if (ret && ret != -ENOENT)
+
+	CKINT_LOG(acvp_search_to_http_type(ret, ACVP_OPTS_DELUP_OE, ctx_opts,
+					   def_oe->acvp_oe_id, &http_type),
+		  "Conversion from search type to HTTP request type failed for OE\n");
+
+	if (http_type == acvp_http_none)
 		goto out;
 
-	/* If we did not find a match, update the module definition */
-	if (ret == -ENOENT) {
-		if (ctx_opts->update_db_entry & ACVP_OPTS_DELUP_OE) {
-			http_type = acvp_http_put;
-		} else if (ctx_opts->delete_db_entry &
-			   (ACVP_OPTS_DELUP_OE | ACVP_OPTS_DELUP_FORCE)) {
-			   http_type = acvp_http_delete;
-		} else if (ctx_opts->register_new_oe) {
-			http_type = acvp_http_post;
-		} else {
-			logger(LOGGER_ERR, LOGGER_C_ANY,
-			       "Definition for OE IDs %u different than found on ACVP server - you need to perform a (re)register operation\n",
-			       def_oe->acvp_oe_id);
-			goto out;
-		}
-	/*
-	 * We only attempt a delete if we have a match between the ACVP server
-	 * DB and our configurations. We do not want to delete unknown
-	 * definitions. Yet, if we are forced to perform the delete, we will
-	 * do that.
-	 */
-	} else if (ctx_opts->delete_db_entry & ACVP_OPTS_DELUP_OE) {
-		http_type = acvp_http_delete;
-	}
-
-	if (http_type) {
-		char url[ACVP_NET_URL_MAXLEN];
-
-		CKINT(acvp_create_url(NIST_VAL_OP_OE, url, sizeof(url)));
-		CKINT(acvp_oe_register_oe(testid_ctx, def_oe, url, sizeof(url),
-					  http_type));
-	}
+	CKINT(acvp_create_url(NIST_VAL_OP_OE, url, sizeof(url)));
+	CKINT(acvp_oe_register_oe(testid_ctx, def_oe, url, sizeof(url),
+				  http_type));
 
 out:
 	return ret;
@@ -781,7 +820,8 @@ static int acvp_oe_validate_all_oe(const struct acvp_testid_ctx *testid_ctx,
 	const struct acvp_ctx *ctx = testid_ctx->ctx;
 	const struct acvp_opts_ctx *ctx_opts = &ctx->options;
 	int ret;
-	char url[ACVP_NET_URL_MAXLEN], queryoptions[256], oestr[128];
+	char oe_name[FILENAME_MAX - 500], url[ACVP_NET_URL_MAXLEN],
+	     queryoptions[FILENAME_MAX], oestr[FILENAME_MAX - 400];
 
 	logger_status(LOGGER_C_ANY,
 		      "Searching for operational environment reference - this may take time\n");
@@ -789,8 +829,8 @@ static int acvp_oe_validate_all_oe(const struct acvp_testid_ctx *testid_ctx,
 	CKINT(acvp_create_url(NIST_VAL_OP_OE, url, sizeof(url)));
 
 	/* Set a query option consisting of the OE name */
-	CKINT(bin2hex_html(def_oe->oe_env_name,
-			   strlen(def_oe->oe_env_name),
+	CKINT(acvp_oe_generate_oe_string(def_oe, oe_name, sizeof(oe_name)));
+	CKINT(bin2hex_html(oe_name, (uint32_t)strlen(oe_name),
 			   oestr, sizeof(oestr)));
 	snprintf(queryoptions, sizeof(queryoptions), "name[0]=contains:%s",
 		 oestr);
@@ -801,6 +841,8 @@ static int acvp_oe_validate_all_oe(const struct acvp_testid_ctx *testid_ctx,
 	/* Our vendor data does not match any vendor on ACVP server */
 	if (!ret) {
 		if (ctx_opts->register_new_oe) {
+			CKINT(acvp_create_url(NIST_VAL_OP_OE, url,
+					      sizeof(url)));
 			CKINT(acvp_oe_register_oe(testid_ctx, def_oe, url,
 						  sizeof(url), acvp_http_post));
 		} else {
@@ -845,8 +887,8 @@ int acvp_oe_handle(const struct acvp_testid_ctx *testid_ctx)
 	if (req_details->dump_register) {
 		char url[ACVP_NET_URL_MAXLEN];
 
-		CKINT(acvp_create_url(NIST_VAL_OP_DEPENDENCY, url,
-				      sizeof(url)));
+		CKINT_ULCK(acvp_create_url(NIST_VAL_OP_DEPENDENCY, url,
+					   sizeof(url)));
 		acvp_oe_register_dep(testid_ctx, def_oe, ACVP_OE_DEP_TYPE_PROC,
 				     acvp_http_post);
 		acvp_oe_register_dep(testid_ctx, def_oe, ACVP_OE_DEP_TYPE_SW,
